@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 
-# Copyright (c) 2009-2012 - Simon Conseil
+# Copyright (c) 2009-2013 - Simon Conseil
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -20,24 +20,24 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-"""
-Prepare images: resize images, and create thumbnails with some options
-(squared thumbs, ...).
-"""
-
 from __future__ import division
 
 import logging
 import os
-
+import sys
+from exceptions import IOError
 from PIL import Image as PILImage
-from PIL import ImageDraw, ImageOps
+from PIL import ImageDraw, ImageOps, ImageFile
+
+# EXIF specs Orientation constant
+EXIF_ORIENTATION_TAG = 274
 
 
-class Image:
-    """ Image container
+class Image(object):
+    """Image container
 
-    resize, thumbnail, ...
+    Prepare images: resize images, and create thumbnails with some options
+    (squared thumbs, ...).
 
     :param filename: path to an image
 
@@ -46,13 +46,43 @@ class Image:
     def __init__(self, filename):
         self.filename = filename
         self.imgname = os.path.split(filename)[1]
-        self.img = PILImage.open(filename)
+        self.logger = logging.getLogger(__name__)
 
-    def save(self, filename, quality=90):
-        self.img.save(filename, quality=quality)
+        with open(filename, 'rb') as fp:
+            self.img = PILImage.open(fp)
+            self.img.load()
+
+        exif = self.img._getexif()
+        if exif:
+            # see: http://www.impulseadventure.com/photo/exif-orientation.html
+            orientation = exif.get(EXIF_ORIENTATION_TAG)
+            rotate_map = {3: 180, 6: -90, 8: 90}
+            rotation = rotate_map.get(orientation)
+            if rotation:
+                self.img = self.img.rotate(rotation)
+
+    def save(self, filename, **kwargs):
+        """Save the image.
+
+        Pass a dict with PIL options (quality, optimize, progressive). PIL can
+        have problems saving large JPEGs if MAXBLOCK isn't big enough, So if
+        we have a problem saving, we temporarily increase it. See
+        http://github.com/jdriscoll/django-imagekit/issues/91
+
+        """
+        try:
+            with quiet():
+                self.img.save(filename, "JPEG", **kwargs)
+        except IOError:
+            old_maxblock = ImageFile.MAXBLOCK
+            ImageFile.MAXBLOCK = self.img.size[0] * self.img.size[1]
+            try:
+                self.img.save(filename, "JPEG", **kwargs)
+            finally:
+                ImageFile.MAXBLOCK = old_maxblock
 
     def resize(self, size):
-        """ Resize the image
+        """Resize the image.
 
         - check if the image format is portrait or landscape and adjust `size`.
         - compute the width and height ratio, and keep the min to resize the
@@ -93,20 +123,18 @@ class Image:
         self.img.save(filename, quality=quality)
 
 
-def copy_exif(srcfile, dstfile):
-    "Copy the exif metadatas from src to dest images"
+class quiet(object):
+    """A context manager for suppressing the stderr activity of PIL's C
+    libraries. Based on http://stackoverflow.com/a/978264/155370
 
-    logger = logging.getLogger(__name__)
+    """
+    def __enter__(self):
+        self.stderr_fd = sys.__stderr__.fileno()
+        self.null_fd = os.open(os.devnull, os.O_RDWR)
+        self.old = os.dup(self.stderr_fd)
+        os.dup2(self.null_fd, self.stderr_fd)
 
-    import pyexiv2
-
-    src = pyexiv2.ImageMetadata(srcfile)
-    dst = pyexiv2.ImageMetadata(dstfile)
-    src.read()
-    dst.read()
-    try:
-        src.copy(dst)
-    except:
-        logger.error("metadata not copied for %s.", srcfile)
-        return
-    dst.write()
+    def __exit__(self, *args, **kwargs):
+        os.dup2(self.old, self.stderr_fd)
+        os.close(self.null_fd)
+        os.close(self.old)
